#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "smtap.h"
#include "tap.h"
#include "operations.h"

unsigned char try_read_int_argument(int argc, char* argv[], char* argName, int* result) {
    for (int i = 0; i < argc - 1; i++) {
        if (!strcmp(argv[i], argName)) {
            *result = atoi(argv[i + 1]);
            return 1;
        }
    }

    return 0;
}

unsigned char try_read_string_argument(int argc, char* argv[], char* argName, char** result) {
    for (int i = 0; i < argc - 1; i++) {
        if (!strcmp(argv[i], argName)) {
            *result = argv[i + 1];
            return 1;
        }
    }
    return 0;
}

void print_program_info() {
    printf("smtap v1 - a ZX Spectrum TAP file manager (written by Sebastian Mihai)\n");
    printf("Usage:\n");
    printf("        smtap -switch value\n");
    printf("\n");
    printf("Switches:\n");
    printf("  -tap <TAP file>     loads the specified TAP file\n");
    printf("                      example:    -tap rastan.tap\n");
    printf("  -extract <slot>     extracts the block in the specified\n");
    printf("                      0-based slot in the loaded TAP file\n");
    printf("                      example:    -extract 3\n");
    printf("  -insert <slot>      inserts a TAP file in the specified 0-based slot of the\n");
    printf("                      loaded TAP file\n");
    printf("                      Note: to append, use location N+1, where N is the last\n");
    printf("                            slot of the receiving TAP file\n");
    printf("                            e.g.: TAP file has 3 slots; to append, specify 3\n");
    printf("                      example:    -insert 4\n");
    printf("  -insertfile <TAP file>\n");
    printf("                      specifies the file to be inserted\n");
    printf("                      this can be a TAP file, or a single TAP file block\n");
    printf("  -outputfile <file>  output will be written to this file\n");
    printf("\n");
}

int main(int argc, char* argv[]) {
    print_program_info();

    e_RunMode runMode = NoTap_InfoOnly;
    int extractIndex;
    int insertIndex;
    char* tapFileName = NULL;

    if (!try_read_string_argument(argc, argv, "-tap", &tapFileName)) {
        return;
    }

    // a TAP file has been specified, so an operation might be defined
    runMode = Tap_Info;

    // find -extract
    if (try_read_int_argument(argc, argv, "-extract", &extractIndex)) {
        runMode = Extract;
    }

    // find -insert
    if (try_read_int_argument(argc, argv, "-insert", &insertIndex)) {
        runMode = Insert;
    }

    if (runMode == NoTap_InfoOnly) {
        // no TAP file was specified, so we just exit (having printed program info)
        return 0;
    }

	tap_start();
    if (!tap_load(tapFileName)) {
        tap_destroy();
        return 0;
    }
	
    if (runMode == Tap_Info) {
        // print TAP information and exit
        tap_destroy();
        return 0;
    }

    // if we get here, we loaded the TAP successfully, and we have to
    // perform an operation on it

    switch (runMode)
    {
    case Extract:
    {
        // we need the output parameter to have been specified
        char* outputFileName = NULL;
        if (!try_read_string_argument(argc, argv, "-outputfile", &outputFileName)) {
            printf("Error: -outputfile must be specified when -extract is specified\n");
            tap_destroy();
            return 0;
        }
        operations_extract_block(outputFileName, extractIndex);
        break;
    }
    case Insert:
    {
        // we need the output parameter to have been specified
        char* outputFileName = NULL;
        if (!try_read_string_argument(argc, argv, "-outputfile", &outputFileName)) {
            printf("Error: -outputfile must be specified when -insert is specified\n");
            tap_destroy();
            return 0;
        }

        // we need the insert data parameter to have been specified
        char* insertFileName = NULL;
        if (!try_read_string_argument(argc, argv, "-insertfile", &insertFileName)) {
            printf("Error: -insertfile must be specified when -insert is specified\n");
            tap_destroy();
            return 0;
        }

        operations_insert(insertFileName, outputFileName, insertIndex);
        break;
    }
    default:
        break;
    }
    
    tap_destroy();
    return 0;
}
