#ifndef __TAP____H__
#define __TAP____H__

enum tapHeaderType { Program = 0, NumberArray = 1, CharacterArray = 2, Code = 3 };
enum tapBlockType { Header = 0x00, Data = 0xFF };

// properties exposed from the data area of a block, when that block is of Header type
struct tapHeaderBlock {
    enum tapHeaderType type;
    char* filename;
    unsigned short dataBlockLength;
    unsigned short parameter1;
    unsigned short parameter2;
};

// a single TAP block
struct tapBlock {
    enum tapBlockType type;
    struct tapHeaderBlock* headerInfo;  // non-NULL when type is Header
    unsigned short blockDataSize;   // whatever the first two bytes of the TAP header said
    unsigned short blockRawDataSize; // excluding ZX Spectrum block's type and checksum bytes
    unsigned char flagByte;         // helps differentiate between 0xFF data and
                            //     headerless (seemingly any value except
                            //     0x00 and 0xFF)
    const char* flagByteHumanReadable;

    unsigned char* tapBlockBytes;       // points to first size byte
    unsigned int tapBlockOffset;      // within TAP file
    unsigned int tapBlockEndOffset;   // within TAP file
    unsigned short tapBlockSize;        // including size bytes

    unsigned short blockIndex;
};

void tap_start();
void tap_destroy();

unsigned short tap_get_block_count();
// caller does not own pointer
struct tapBlock* tap_get_block(int index);
// caller does not own pointer
char* tap_get_filename();
unsigned char tap_load(char* filename);
unsigned char tap_is_file_loaded();

#endif
